#define MAXSECTORS 1024
#define MAXWALLS 4096
#define MAXSPRITES 4096
#define MAXTILES 4096
#define MAXSTATUS 1024
#define MAXANIMATES 512
#define NUMPALOOKUPS 32
#define NUMOPTIONS 6
#define XDIM 320
#define YDIM 200

	//Make all variables in BUILD.H defined in the ENGINE,
	//and externed in GAME
#ifdef ENGINE
	#define EXTERN
#else
	#define EXTERN extern
#endif

//ceilingstat, bit 0: 1 = ceiling parallaxing sky, 0 = not
//             bit 1: 1 = ceiling groudraw, 0 = not (not supported yet)
//             bit 2: 1 = Vertical panning, 0 = Horizontal panning
//             bit 3: 1 = pixel-smooshing mode, 0 = tile-expanding mode
//floorstat,   bit 0: 1 = floor parallaxing sky, 0 = not (not supported yet)
//             bit 1: 1 = floor groudraw, 0 = not
//             bit 2: 1 = Vertical panning, 0 = Horizontal panning
//             bit 3: 1 = pixel-smooshing mode, 0 = tile-expanding mode

static struct sectortype
{
	short unsigned int wallptr, wallnum;
	char ceilingstat, ceilingpanning;
	signed char ceilingshade;
	long ceilingz;
	short ceilingpicnum, ceilingheinum;
	char floorstat, floorpanning;
	signed char floorshade;
	long floorz;
	short floorpicnum, floorheinum;
	long tag;
};

//cstat, bit 0: 1 = clip xoring, 0 = not
//       bit 1: 1 = bottoms of invisible walls swapped, 0 = not
//       bit 2: 1 = align picture on bottom (for doors), 0 = top
//       bit 3: 1 = x-flipped, 0 = normal
//       bit 4: 1 = masking wall, 0 = not
//       bit 5: 1 = 1-way wall, 0 = not
//       bit 6: 1 = Vertical panning, 0 = Horizontal panning

static struct walltype
{
	long x, y;
	short point2;
	char cstat;
	signed char shade;
	unsigned char xrepeat, yrepeat, panning;
	short picnum, overpicnum;
	short nextsector1, nextwall1;
	short nextsector2, nextwall2;
	long tag;
};

//cstat, bit 0: 1 = Sprite blocks bullets (ex: hitscan) and you, 0 = not
//              Yes.  That's right.  This is the bit that tells whether
//              hitscan is supposed to stop at this sprite (1) or just
//              ignore it (0).
//       bit 1: reserved
//       bit 2: 1 = x-flipped, 0 = normal

static struct spritetype
{
	long x, y, z;
	char cstat;
	signed char shade;
	unsigned char xrepeat, yrepeat;
	short picnum, ang, vel, zvel, owner;
	short sectnum, statnum;
	long tag;
	char *extra;
};

EXTERN struct sectortype sector[MAXSECTORS];
EXTERN struct walltype wall[MAXWALLS];
EXTERN struct spritetype sprite[MAXSPRITES];

EXTERN long posx, posy, posz;
EXTERN short int ang, cursectnum;
EXTERN volatile char keystatus[256];

EXTERN char option[NUMOPTIONS];
EXTERN short startumost[XDIM], startdmost[XDIM];
EXTERN short int sintable[2048];
EXTERN short numsectors, numwalls;
EXTERN volatile long clockspeed, lockspeed, totalclock, numframes;
EXTERN long vel, svel, hvel, angvel, newposz;
EXTERN char visibility, parallaxtype;
EXTERN long heightoffceiling, heightofffloor, gravity;

EXTERN short headspritesect[MAXSECTORS+1], headspritestat[MAXSTATUS+1];
EXTERN short prevspritesect[MAXSPRITES], prevspritestat[MAXSPRITES];
EXTERN short nextspritesect[MAXSPRITES], nextspritestat[MAXSPRITES];

	//These 3 variables tell you whether you're ready to use something.
	//(Looking at it and close enough)  If you are ready, the variable will
	//be >= 0 representing the number of the actual sector, wall or sprite.
	//If not, then the variable will be -1.
EXTERN short neartagsector, neartagwall, neartagsprite;

	//These variables are for animating x, y, or z-coordinates of sectors,
	//walls, or sprites (They are NOT to be used for changing the [].picnum's)
	//See the setanimation(), and getanimategoal() functions for more details.
EXTERN long *animateptr[MAXANIMATES], animategoal[MAXANIMATES];
EXTERN long animatevel[MAXANIMATES], animatecnt;

EXTERN short tilesizx[MAXTILES], tilesizy[MAXTILES];
EXTERN long numtiles, picanm[MAXTILES], waloff[MAXTILES];


/*************************************************************************
POSITION VARIABLES:

		POSX is your x - position ranging from 0 to 65535
		POSY is your y - position ranging from 0 to 65535
			(the length of a side of the grid in EDITBORD would be 1024)
		POSZ is your z - position (height) ranging from 0 to 65535, 0 highest.
		ANG is your angle ranging from 0 to 2047.  Instead of 360 degrees, or
			 2 * PI radians, I use 2048 different angles, so 90 degrees would
			 be 512 in my system.
		HEIGHTOFFCEILING - height off ceiling. Default is 32<<8.
		HEIGHTOFFFLOOR - height off floor.  Default is 32<<8.
		GRAVITY - can be 0 or negative. Default is 98.

KEYBOARD READING (KEYSTATUS):

		- Up / Down status of all keys on keyboard
		For example, keystatus[1] happens to be the state of the ESCAPE KEY

		if keystatus[1] = 0 then the escape key is not pressed while
		if keystatus[1] > 0 then the esacpe key is pressed

		This is good for checking if multiple keys are pressed at the same time.
		One disadvantage of this method of reading the keyboard is that the
	keystatus[x] stays > 0 until you release the key.  If you want to be
	certain that whatever is done once when the key is pressed once, you
	may want to set keystatus[x] = 0 right after you sense
	keystatus[x] > 0.

VISIBILITY:

		- for special effects such as a gun shooting.
		A value from 5 to around 15.
		13 is normal, the lower the number, the darker.

SPRITE VARIABLES:

	EXTERN short headspritesect[MAXSECTORS+1], headspritestat[MAXSTATUS+1];
	EXTERN short prevspritesect[MAXSPRITES], prevspritestat[MAXSPRITES];
	EXTERN short nextspritesect[MAXSPRITES], nextspritestat[MAXSPRITES];

	Example: if the linked lists look like the following:
		 Ŀ
		       Sector lists:               Status lists:               
		 Ĵ
		   Sector0:  4, 5, 8             Status0:  2, 0, 8             
		   Sector1:  16, 2, 0, 7         Status1:  4, 5, 16, 7, 3, 9   
		   Sector2:  3, 9                                              
		 
	Notice that each number listed above is shown exactly once on both the
		left and right side.  This is because any sprite that exists must
		be in some sector, and must have some kind of status that you define.


Coding example #1:
	To go through all the sprites in sector 1, the code can look like this:

		sectnum = 1;
		i = headspritesect[sectnum];
		while (i != -1)
		{
			nexti = nextspritesect[sectnum];

			//your code goes here
			//ex: printf("Sprite %d is in sector %d\n",i,sectnum);

			i = nexti;
		}

Coding example #2:
	To go through all sprites with status = 1, the code can look like this:

		statnum = 1;        //status 1
		i = headspritestat[statnum];
		while (i != -1)
		{
			nexti = nextspritestat[statnum];

			//your code goes here
			//ex: printf("Sprite %d has a status of 1 (active)\n",i,statnum);

			i = nexti;
		}

			 insertsprite(short sectnum, short statnum);
			 deletesprite(short spritenum);
			 changespritesect(short spritenum, short newsectnum);
			 changespritestat(short spritenum, short newstatnum);

DOOR VARIABLES:
		DOORSECTOR is -1 except when you are in the position to open a door.
			When you are in that position (at the proper distance and angle),
			then DOORSECTOR equals the sector of the doorsector that can be
			moved by pressing space bar.
		DOORCNT is the number of doors (0 to DOORCNT-1) that are active.
		DOORSECTORLIST[MAXACTIVEDOORS] is an array of all the sector numbers
			of doors that are currently moving.
		DOORSECTORGOALZ[MAXACTIVEDOORS] is an array of all the goal z
			coordinates that each ceiling of the door is trying to reach.

		Example: To program a door to close (Given: thedoorsectnum), do this:

			doorsectorlist[doorcnt] = thedoorsectnum;
			doorsectorgoalz[doorcnt] = sector[thedoorsectnum].floorz;
			doorcnt++;

TILE VARIABLES:
		NUMTILES - the number of tiles found TILES.DAT.
		TILESIZX[MAXTILES] - simply the x-dimension of the tile number.
		TILESIZY[MAXTILES] - simply the y-dimension of the tile number.
		WALOFF[MAXTILES] - the actual 32-bit offset pointing to the top-left
								 corner of the tile.
		PICANM[MAXTILES] - flags for animating the tile.

TIMING VARIABLES:
		TOTALCLOCK - When the engine is initialized, TOTALCLOCK is set to zero.
			From then on, it is incremented 120 times a second by 1.  That
			means that the number of seconds elapsed is totalclock / 120.
		CLOCKSPEED - The number of tics (120/second) that have passed while
			drawing the last frame. (use this variable to make things run the
			same speed on different computers.  Also use totalclock)
		NUMFRAMES - The number of times the draw3dscreen function was called
			since the engine was initialized.  This helps to determine frame
			rate.  (Frame rate = numframes * 120 / totalclock.)

OTHER VARIABLES:

		STARTUMOST[320] is an array of the highest y-coordinates on each column
				that my engine is allowed to write to.  You need to set it only
				once.
		STARTDMOST[320] is an array of the lowest y-coordinates on each column
				that my engine is allowed to write to.  You need to set it only
				once.
		SINTABLE[2048] is a sin table with 2048 angles rather than the
			normal 360 angles for higher precision.  Also since SINTABLE is in
			all integers, the range is multiplied by 16383, so instead of the
			normal -1<sin(x)<1, the range of sintable is -16383<sintable[]<16383
			If you use this sintable, you can possibly speed up your code as
			well as save space in memory.  If you plan to use sintable, 2
			identities you may want to keep in mind are:
				sintable[ang&2047]       = sin(ang * (3.141592/1024)) * 16383
				sintable[(ang+512)&2047] = cos(ang * (3.141592/1024)) * 16383
		OPTION[NUMOPTIONS] - this array is modified by the SETUP.EXE program.
			Right now, option[0] is the graphics mode (0-3)
						  option[1] is the digitized sound (0-5)
						  option[2] is the music (0-1)
						  option[3] is the input type (0-1)
						  option[4] is the serial port (0-2)
			Please look at the SETUP program - the array is set up just like you
				would expect.
		NUMSECTORS - the total number of existing sectors.  Modified every time
			you call the loadboard function.
***************************************************************************/
