#include <string.h>
#include <stdlib.h>
#include "build.h"
#include "names.h"

/***************************************************************************
	KEN'S TAG DEFINITIONS:      (Please define your own tags for your games)

 sector[?].tag = 0    Normal sector
 sector[?].tag = 1    Being in sector with tag 1 opens all door sectors
								 with tag 2 and turns all sprites in sectors with
								 tag 3 to active brown monsters. (Walk in the
								 middle of the animated floor tiles in the
								 beginning of DOOM1.DAT)
 sector[?].tag = 2    A sector that will be opened as soon as you first
								 stand in a sector with tag 1.
 sector[?].tag = 3    A sector where all the sprites will be converted to
								 brown monsters as soon as you first step into a
								 sector with tag 3.
 sector[?].tag = 4    A sector where you are put closer to the floor
								 (such as the slime in DOOM1.DAT)
 sector[?].tag = 5    A sector with no gravity.
 sector[?].tag = 6    A normal door - instead of pressing D, you tag the
								 sector with a 6.  The reason I make you edit doors
								 this way is so that can program the doors
								 yourself.
 sector[?].tag = 7    A door the goes down to open.
 sector[?].tag = 8    A door that opens horizontally in the middle.
 sector[?].tag = 9    A sliding door that opens vertically in the middle.
								 -Example of the advantages of not using BSP tree.
 sector[?].tag = 10   A warping sector with floor and walls that shade.
 sector[?].tag = 11   A sector will all walls that do X-panning.


	wall[?].tag = 0    Normal wall
	wall[?].tag = 1    Y-panning wall


 sprite[?].tag = 0    Normal sprite
 sprite[?].tag = 1    If you press space bar on an AL, and the AL is tagged
								 with a 1, he will turn evil.


KEN'S STATUS DEFINITIONS:  (Please define your own statuses for your games)
 status = 0            Inactive sprite
 status = 1            Active sprite
 status = 2            Monster that becomes active only when it sees you
 status = MAXSTATUS    Non-existent sprite (this will be true for your
														  code also)
**************************************************************************/

	//Here are some nice in-line assembly pragmas that make life easier.
	//(at least for Ken)
#pragma aux setvmode =\
	"int 0x10",\
	parm [eax]\

#pragma aux getlong =\
	"mov eax, dword ptr [eax]",\
	parm [eax]\

main(short int argc,char **argv)
{
	char boardfilename[13], buffer[80], ch;
	short startwall, endwall, doubvel, wallfind[2], dasector;
	long i, j, k, s, nexti, dax, day, daz, score, gotsprite, good, cnt;
	long hitx, hity, hitsect, hitwall, hitsprite, health, dax2, day2, daz2;
	short warpsectorlist[16], warpsectorcnt, warpsectorflag;
	short xpanningsectorlist[16], xpanningsectorcnt;
	short ypanningwalllist[64], ypanningwallcnt;
	short floorpanninglist[16], floorpanningcnt;

	char syncbuffer[32], synccnt, synclen, numplayers;       //Multiplayer
	short otherplayer, mefirst, dat, nukes[2];               //Variables
	long lastyourpositionsend;

	initengine();
	parallaxtype = 2;
	initsb(165);

//If there is an argument on the command line, load that board
//else load the default board, "boards.map"

	if (argc >= 2)
	{
		strcpy(&boardfilename,argv[1]);
		if(strchr(boardfilename,'.') == 0)
			strcat(boardfilename,".map");
	}
	else
		strcpy(&boardfilename,"nukeland.map");
	if (loadboard(boardfilename) == -1)
	{
		printf("Board not found\n");
		return(-1);
	}

	loadpics("tiles000.art");

		//Prepare variables just after loading board
	for(i=0;i<MAXSPRITES;i++)
		if (sprite[i].statnum < MAXSTATUS)   //That is, if sprite exists
		{
			switch(sprite[i].picnum)
			{
				case SKELETON:               //All cases here put the sprite
				case BROWNMONSTER:           //on status list 1 (active)
					changespritestat(i,2);
					break;
			}
			switch(sprite[i].picnum)
			{
				case SKELETON:               //Only these sprites will be
				case AL:                     //sensitive to the hitscan function
				case BROWNMONSTER:
				case EVILAL:
				case DOOMGUY:
					sprite[i].cstat |= 1;     //Set the hitscan sensitivity bit
					break;
			}
		}

	warpsectorflag = 0;
	warpsectorcnt = 0;      //Make a list of warping sectors (16 max)
	xpanningsectorcnt = 0;  //Make a list of wall x-panning sectors (16 max)
	floorpanningcnt = 0;
	for(i=0;i<numsectors;i++)
	{
		if (sector[i].tag == 4) floorpanninglist[floorpanningcnt++] = i;
		if (sector[i].tag == 10) warpsectorlist[warpsectorcnt++] = i;
		if (sector[i].tag == 11) xpanningsectorlist[xpanningsectorcnt++] = i;
	}
	ypanningwallcnt = 0;
	for(i=0;i<numwalls;i++)
	{
		if (wall[i].tag == 1)
		{
			ypanningwalllist[ypanningwallcnt++] = i;
			wall[i].cstat |= 64;
		}
	}

	setgamemode();

	loadmusic("maxosong.ksm");
	musicon();

	for (i=0;i<XDIM;i++)                      //define 3d picture boundaries
	{
		startumost[i] = 0;
		startdmost[i] = YDIM-32;
	}

	score = 0L;

	permanentwritesprite(0,YDIM-32,STATUSBAR,0);

	numplayers = 1;
	if (option[4] > 0)
	{
		otherplayer = insertsprite(cursectnum,0);
		sprite[otherplayer].x = posx;
		sprite[otherplayer].y = posy;
		sprite[otherplayer].z = posz+(32<<8);
		sprite[otherplayer].cstat = 1;            //Hitscan should hit opponent
		sprite[otherplayer].picnum = DOOMGUY;
		sprite[otherplayer].shade = 0;
		sprite[otherplayer].xrepeat = 64;
		sprite[otherplayer].yrepeat = 64;
		sprite[otherplayer].ang = ang;
		sprite[otherplayer].vel = 384;
		sprite[otherplayer].zvel = 0;
		sprite[otherplayer].owner = -1;           //-1 is you
		sprite[otherplayer].tag = 0L;

		nukes[0] = 0L;
		nukes[1] = 0L;
		lastyourpositionsend = totalclock;
		synccnt = 0;
		numplayers = 1;
		health = 100;
		comsend(0L,1L);
	}

	while ((keystatus[1] == 0) && (sector[cursectnum].tag != 65535))
	{
		if (option[4] > 0)
		{
			while ((dat = comgetchar()) >= 0)
			{
				syncbuffer[synccnt++] = (char)dat;
				if (synccnt == 1)
				{
					switch(dat)
					{
						case 0: synclen = 1; break;
						case 1: synclen = 1; break;
						case 2: synclen = 15; break;
						case 3: synclen = 1; break;
						case 4: synclen = 13; break;
						case 5: synclen = 8; break;
						case 6: synclen = 3; break;
						case 7: synclen = 3; break;
						case 8: synclen = 1; break;
						case 254:
							playback();
							synccnt = 0;
							synclen = 255;
							break;
						case 255: synclen = 1; break;
					}
				}
				if (synccnt >= synclen)
				{
					switch(syncbuffer[0])
					{
						case 0:
							comsend(1L,1L);
							numplayers = 2;
							break;
						case 1:
							numplayers = 2;
							break;
						case 2:
							sprite[otherplayer].x = getlong((long)&syncbuffer[1]);
							sprite[otherplayer].y = getlong((long)&syncbuffer[5]);
							sprite[otherplayer].z = getlong((long)&syncbuffer[9])+(32<<8);
							sprite[otherplayer].ang = syncbuffer[13]+(syncbuffer[14]<<8);
							movesprite(otherplayer,0L,0L,0L);
							break;
						case 3:
							wsay("shoot2.wav",1023/((((sprite[otherplayer].x-posx)*(sprite[otherplayer].x-posx)+(sprite[otherplayer].y-posy)*(sprite[otherplayer].y-posy))>>21)+4));
							j = insertsprite(sprite[otherplayer].sectnum,1);
							sprite[j].x = sprite[otherplayer].x;
							sprite[j].y = sprite[otherplayer].y;
							sprite[j].z = sprite[otherplayer].z-(32<<8)+(8<<8);
							sprite[j].cstat = 0;                 //Hitscan doesn't hit other bullets
							sprite[j].picnum = BULLET;
							sprite[j].shade = 0;
							sprite[j].xrepeat = 64;
							sprite[j].yrepeat = 64;
							sprite[j].ang = sprite[otherplayer].ang;
							sprite[j].vel = 384;
							sprite[j].zvel = 0;
							sprite[j].owner = otherplayer;      //-1 is you
							sprite[j].tag = 0L;
							break;
						case 4:
							hitx = getlong((long)&syncbuffer[1]);
							hity = getlong((long)&syncbuffer[5]);
							hitsect = getlong((long)&syncbuffer[9]);
							wsay("shoot2.wav",1023/((((hitx-posx)*(hitx-posx)+(hity-posy)*(hity-posy))>>21)+4));
							j = insertsprite(hitsect,0);
							sprite[j].x = hitx;
							sprite[j].y = hity;
							sprite[j].z = sprite[otherplayer].z-(32<<8)+(8<<8);
							sprite[j].cstat = 0;                 //Hitscan doesn't hit smoke on wall
							sprite[j].picnum = BULLET;
							sprite[j].shade = 0;
							sprite[j].xrepeat = 64;
							sprite[j].yrepeat = 64;
							sprite[j].ang = sprite[otherplayer].ang;
							sprite[j].vel = 0;
							sprite[j].zvel = 0;
							sprite[j].owner = otherplayer;      //-1 is you
							sprite[j].tag = 0L;
							break;
						case 5:
							j = syncbuffer[2]+(syncbuffer[3]<<8);
							k = getlong((long)&syncbuffer[4]);
							if ((syncbuffer[1]&1) == 0)
								setanimation(&sector[j].ceilingz,k,128);
							else
								setanimation(&sector[j].floorz,k,128);
							if ((syncbuffer[1]&2) > 0)
								wsay("updowndr.wav",1023/((((sprite[otherplayer].x-posx)*(sprite[otherplayer].x-posx)+(sprite[otherplayer].y-posy)*(sprite[otherplayer].y-posy))>>21)+4));
							break;
						case 6:
							j = syncbuffer[1]+(syncbuffer[2]<<8);
							deletesprite(j);
							wsay("getstuff.wav",1023/((((sprite[otherplayer].x-posx)*(sprite[otherplayer].x-posx)+(sprite[otherplayer].y-posy)*(sprite[otherplayer].y-posy))>>21)+4));
							break;
						case 7:
							nukes[0] = syncbuffer[1]+(syncbuffer[2]<<8);
							wsay("death.wav",1023/((((sprite[otherplayer].x-posx)*(sprite[otherplayer].x-posx)+(sprite[otherplayer].y-posy)*(sprite[otherplayer].y-posy))>>21)+4));
							break;
						case 8:
							wsay("ouch.wav",1023/((((sprite[otherplayer].x-posx)*(sprite[otherplayer].x-posx)+(sprite[otherplayer].y-posy)*(sprite[otherplayer].y-posy))>>21)+4));
							break;
						case 255:
							numplayers = 1;
							sprite[otherplayer].picnum = AL;
							break;
					}
					synccnt = 0;
				}
			}
		}

		if (numplayers == 2)
		{
				// Calculate which of the 8 angles of the sprite to draw (0-7)
			i = ((((sprite[otherplayer].ang+3072+128-ang)&2047)>>8)&7);
			if (i <= 4)
			{
				sprite[otherplayer].picnum = DOOMGUY + 4 * i;
				sprite[otherplayer].cstat &= (255-4);   //clear x-flipping bit
			}
			else
			{
				sprite[otherplayer].picnum = DOOMGUY + 4 * (8-i);
				sprite[otherplayer].cstat |= 4;     //set x-flipping bit
			}
		}
		draw3dscreen();

		if (totalclock < 120)
			overwritesprite(160L,100L,DEMOSIGN,0L,1);

		if (numplayers == 2)
		{
			if (totalclock >= lastyourpositionsend+16-((option[5]>=2)<<3))
			{
				comsend(2L,1);
				comsend(posx,4);
				comsend(posy,4);
				comsend(posz,4);
				comsend((long)ang,2);
				lastyourpositionsend = totalclock;
			}

			sprintf(&buffer,"Health: %3d",health);
			printext(160-(strlen(buffer)<<2),176,buffer,ALPHABET,80);
			sprintf(&buffer,"Nukes: You-%d, Other-%d",nukes[0],nukes[1]);
			printext(160-(strlen(buffer)<<2),184,buffer,ALPHABET,80);
		}

		sprintf(&buffer,"Score:%ld",score);
		printext(160-(strlen(buffer)<<2),192,buffer,ALPHABET,80);


			//Game input
		if (keystatus[0x1d] > 0)   //Shooting (rocket) - LeftCtrl key
		{
			keystatus[0x1d] = 0;
			wsay("shoot2.wav",255);
			j = insertsprite(cursectnum,1);
			sprite[j].x = posx;
			sprite[j].y = posy;
			sprite[j].z = posz+(8<<8);
			sprite[j].cstat = 0;        //Hitscan does not hit other bullets
			sprite[j].picnum = BULLET;
			sprite[j].shade = 0;
			sprite[j].xrepeat = 64;
			sprite[j].yrepeat = 64;
			sprite[j].ang = ang;
			sprite[j].vel = 384;
			sprite[j].zvel = 0;
			sprite[j].owner = -1;      //-1 is you
			sprite[j].tag = 0L;

			if (numplayers == 2)
			{
				comsend(3L,1);
			}
		}
		if (keystatus[0x38] > 0)   //Shooting (shotgun) - LeftAlt key
		{
			keystatus[0x38] = 0;
			hitscan(posx,posy,posz,ang,cursectnum,16384,&hitx,&hity,&hitsect,&hitwall,&hitsprite);
			wsay("shoot2.wav",255);
			j = insertsprite(hitsect,0);
			sprite[j].x = hitx;
			sprite[j].y = hity;
			sprite[j].z = posz+(8<<8);
			sprite[j].cstat = 0;        //Hitscan does not hit smoke on wall
			sprite[j].picnum = BULLET;
			sprite[j].shade = 0;
			sprite[j].xrepeat = 64;
			sprite[j].yrepeat = 64;
			sprite[j].ang = ang;
			sprite[j].vel = 0;
			sprite[j].zvel = 0;
			sprite[j].owner = -1;      //-1 is you
			sprite[j].tag = 0L;
				//Sprite starts out with center exactly on wall.
				//This moves it back enough to see it at all angles.
			movesprite(j,-(sintable[(512+ang)&2047]>>7),-(sintable[ang]>>7),0L);
			if (numplayers == 2)
			{
				comsend(4L,1);
				comsend(hitx,4);
				comsend(hity,4);
				comsend(hitsect,4);
			}

		}
		if (keystatus[0x39] > 0)                  //If you press SPACE BAR
		{
			if ((sector[neartagsector].tag == 6) || (sector[neartagsector].tag == 8))    //If the sector in front is a door
			{
				i = getanimationgoal(&sector[neartagsector].ceilingz);
				if (i >= 0)      //If door already moving, reverse its direction
				{
					if (sector[neartagsector].tag == 8)
						daz = ((sector[neartagsector].ceilingz+sector[neartagsector].floorz)>>1);
					else
						daz = sector[neartagsector].floorz;

					if (animategoal[i] == daz)
						animategoal[i] = sector[nextsectorneighborz(neartagsector,sector[neartagsector].floorz,-1,-1)].ceilingz;
					else
						animategoal[i] = daz;
					if (numplayers == 2)
					{
						comsend(5L,1);
						comsend(0L,1);
						comsend(neartagsector,2);
						comsend(animategoal[i],4);
					}
				}
				else      //else insert the door's ceiling on the animation list
				{
					if (sector[neartagsector].ceilingz == sector[neartagsector].floorz)
						daz = sector[nextsectorneighborz(neartagsector,sector[neartagsector].floorz,-1,-1)].ceilingz;
					else
					{
						if (sector[neartagsector].tag == 8)
							daz = ((sector[neartagsector].ceilingz+sector[neartagsector].floorz)>>1);
						else
							daz = sector[neartagsector].floorz;
					}
					if ((j = setanimation(&sector[neartagsector].ceilingz,daz,128)) >= 0)
					{
						if (numplayers == 2)
						{
							comsend(5L,1);
							comsend(0L+2L,1);
							comsend(neartagsector,2);
							comsend(animategoal[j],4);
						}
						wsay("updowndr.wav",255);
					}
				}
				keystatus[0x39] = 0;
			}
			if ((sector[neartagsector].tag == 7) || (sector[neartagsector].tag == 8)) //If the sector in front's elevator
			{
				i = getanimationgoal(&sector[neartagsector].floorz);
				if (i >= 0)      //If elevator already moving, reverse its direction
				{
					if (sector[neartagsector].tag == 8)
						daz = ((sector[neartagsector].ceilingz+sector[neartagsector].floorz)>>1);
					else
						daz = sector[neartagsector].ceilingz;

					if (animategoal[i] == daz)
						animategoal[i] = sector[nextsectorneighborz(neartagsector,sector[neartagsector].ceilingz,1,1)].floorz;
					else
						animategoal[i] = daz;

					if (numplayers == 2)
					{
						comsend(5L,1);
						comsend(1L,1);
						comsend(neartagsector,2);
						comsend(animategoal[i],4);
					}
				}
				else      //else insert the elevator's ceiling on the animation list
				{
					if (sector[neartagsector].floorz == sector[neartagsector].ceilingz)
						daz = sector[nextsectorneighborz(neartagsector,sector[neartagsector].ceilingz,1,1)].floorz;
					else
					{
						if (sector[neartagsector].tag == 8)
							daz = ((sector[neartagsector].ceilingz+sector[neartagsector].floorz)>>1);
						else
							daz = sector[neartagsector].ceilingz;
					}
					if ((j = setanimation(&sector[neartagsector].floorz,daz,128)) >= 0)
					{
						if (numplayers == 2)
						{
							comsend(5L,1);
							comsend(1L+((sector[neartagsector].tag!=8)<<1),1);
							comsend(neartagsector,2);
							comsend(animategoal[j],4);
						}
						wsay("updowndr.wav",255);
					}
				}
				keystatus[0x39] = 0;
			}

			if (sector[neartagsector].tag == 9)
			{
				startwall = sector[neartagsector].wallptr;
				endwall = startwall+sector[neartagsector].wallnum-1;

					//first find center point by averaging all points
				dax = 0L, day = 0L;
				for(i=startwall;i<=endwall;i++)
				{
					dax += wall[i].x;
					day += wall[i].y;
				}
				dax /= (endwall-startwall+1);
				day /= (endwall-startwall+1);

					//find any points with either same x or same y coordinate
					//  as center (dax, day) - should be 2 points found.
				wallfind[0] = -1;
				wallfind[1] = -1;
				for(i=startwall;i<=endwall;i++)
					if ((wall[i].x == dax) || (wall[i].y == day))
					{
						if (wallfind[0] == -1)
							wallfind[0] = i;
						else
							wallfind[1] = i;
					}

				for(j=0;j<2;j++)
				{
					if ((wall[wallfind[j]].x == dax) && (wall[wallfind[j]].y == day))
					{
							//find what direction door should open by averaging the
							//  2 neighboring points of wallfind[0] & wallfind[1].
						i = wallfind[j]-1; if (i < startwall) i = endwall;
						dax2 = ((wall[i].x+wall[wall[wallfind[j]].point2].x)>>1)-wall[wallfind[j]].x;
						day2 = ((wall[i].y+wall[wall[wallfind[j]].point2].y)>>1)-wall[wallfind[j]].y;
						if (dax2 != 0)
						{
							dax2 = wall[wall[wall[wallfind[j]].point2].point2].x;
							dax2 -= wall[wall[wallfind[j]].point2].x;
							setanimation(&wall[wallfind[j]].x,wall[wallfind[j]].x+dax2,4);
							setanimation(&wall[i].x,wall[i].x+dax2,4);
							setanimation(&wall[wall[wallfind[j]].point2].x,wall[wall[wallfind[j]].point2].x+dax2,4);
						}
						else if (day2 != 0)
						{
							day2 = wall[wall[wall[wallfind[j]].point2].point2].y;
							day2 -= wall[wall[wallfind[j]].point2].y;
							setanimation(&wall[wallfind[j]].y,wall[wallfind[j]].y+day2,4);
							setanimation(&wall[i].y,wall[i].y+day2,4);
							setanimation(&wall[wall[wallfind[j]].point2].y,wall[wall[wallfind[j]].point2].y+day2,4);
						}
					}
					else
					{
						i = wallfind[j]-1; if (i < startwall) i = endwall;
						dax2 = ((wall[i].x+wall[wall[wallfind[j]].point2].x)>>1)-wall[wallfind[j]].x;
						day2 = ((wall[i].y+wall[wall[wallfind[j]].point2].y)>>1)-wall[wallfind[j]].y;
						if (dax2 != 0)
						{
							setanimation(&wall[wallfind[j]].x,dax,4);
							setanimation(&wall[i].x,dax+dax2,4);
							setanimation(&wall[wall[wallfind[j]].point2].x,dax+dax2,4);
						}
						else if (day2 != 0)
						{
							setanimation(&wall[wallfind[j]].y,day,4);
							setanimation(&wall[i].y,day+day2,4);
							setanimation(&wall[wall[wallfind[j]].point2].y,day+day2,4);
						}
					}
				}
				wsay("updowndr.wav",255);

				keystatus[0x39] = 0;
			}

			if (sprite[neartagsprite].tag == 1)
			{     //if you're shoving innocent little AL around, he gets mad!
				if (sprite[neartagsprite].picnum == AL)
				{
					sprite[neartagsprite].picnum = EVILAL;
					sprite[neartagsprite].xrepeat = 38;
					sprite[neartagsprite].yrepeat = 38;
				}
				changespritestat(neartagsprite,1);
				keystatus[0x39] = 0;
			}
		}

		if (numplayers == 2)
		{
				 //Record & send a sound to remote computer
			if (keystatus[0x1c] != 0)
			{
				if (totalclock > 240)
				{
					record();
					keystatus[0x1c] = 0;
				}
				else
				{
					keystatus[0x1c] = 0;
				}
			}
		}

		if (keystatus[0x2f] > 0)  // V
		{
			keystatus[0x2f] = 0;
			visibility--;
			if (visibility == 7)
				visibility = 15;
		}
		if (angvel != 0)          //ang += angvel * constant
		{                         //ENGINE calculates angvel for you
			doubvel = lockspeed;
			if (keystatus[0x2a] > 0)  //Lt. shift makes turn velocity 50% faster
				doubvel += (lockspeed>>1);

			ang += ((angvel*doubvel)>>5);
			ang = (ang+2048)&2047;
		}

		doubvel = lockspeed;
		if (keystatus[0x2a] > 0)     //Lt. shift doubles forward velocity
			doubvel += lockspeed;

		if (vel != 0)                   //vel = forward velocity
			clipmove((vel*doubvel)>>3,ang);  //Change posx/posy with ang & clip
		if (svel != 0)                  //svel = strafing velocity
			clipmove((svel*doubvel)>>3,((ang+1536)&2047)); //Change posx/posy with ang & clip
		getsector();

		heightoffceiling = (32<<8);
		heightofffloor = (32<<8);
		if (keystatus[0x1e] > 0)                            //A (stand high)
		{
			heightoffceiling -= (16<<8);
			heightofffloor += (16<<8);
			if ((keystatus[0x2a]|keystatus[0x36]) > 0)       //Either shift key
			{
				heightoffceiling -= (16<<8);
				heightofffloor += (16<<8);
			}
		}
		if (keystatus[0x2c] > 0)                            //Z (stand low)
		{
			heightoffceiling += (16<<8);
			heightofffloor -= (16<<8);
			if ((keystatus[0x2a]|keystatus[0x36]) > 0)       //Either shift key
			{
				heightoffceiling += (16<<8);
				heightofffloor -= (16<<8);
			}
		}

		if (sector[cursectnum].tag == 4)   //slime sector
		{
			clipmove(24,1024);
			heightofffloor -= (28<<8);
		}

		engineinput();

		nextpage();

			//Tag code
		if (sector[cursectnum].tag == 1)
		{
			for(i=0;i<numsectors;i++)
			{
				if (sector[i].tag == 2)
				{
					setanimation(&sector[i].ceilingz,sector[i].floorz-16384,64);
					wsay("updowndr.wav",255);
				}
				if (sector[i].tag == 3)
				{
					j = headspritesect[i];
					while (j != -1)
					{
						k = nextspritesect[j];
						sprite[j].picnum = BROWNMONSTER;
						changespritestat(j,2);
						j = k;
					}
				}
			}
			sector[cursectnum].tag = 0;
		}
		if (sector[cursectnum].tag == 5)   //no gravity sector
			gravity = 0;
		else
			gravity = 98;

		for(i=0;i<warpsectorcnt;i++)
		{
			dasector = warpsectorlist[i];
			j = ((totalclock&127)>>2);
			if (j >= 16) j = 31 - j;
			{
				sector[dasector].ceilingshade = j;
				sector[dasector].floorshade = j;
				startwall = sector[dasector].wallptr;
				endwall = startwall+sector[dasector].wallnum-1;
				for(s=startwall;s<=endwall;s++)
					wall[s].shade = j;
			}
		}
		if (sector[cursectnum].tag == 10)  //warp sector
		{
			if (warpsectorflag == 0)
			{
				wsay("warp.wav");

				for(i=0;i<warpsectorlist;i++)
				{
					if (warpsectorlist[i] == cursectnum)
					{
						if (i == warpsectorcnt-1)
							cursectnum = warpsectorlist[0];
						else
							cursectnum = warpsectorlist[i+1];
						break;
					}
				}
					//Find center of sector
				startwall = sector[cursectnum].wallptr;
				endwall = startwall+sector[cursectnum].wallnum-1;
				dax = 0L, day = 0L;
				for(s=startwall;s<=endwall;s++)
				{
					dax += wall[s].x, day += wall[s].y;
					if (wall[s].nextsector1 >= 0)
						i = s;
				}
				posx = dax / (endwall-startwall+1);
				posy = day / (endwall-startwall+1);
				posz = sector[cursectnum].floorz - heightofffloor;
				getsector();
				dax = ((wall[i].x+wall[wall[i].point2].x)>>1);
				day = ((wall[i].y+wall[wall[i].point2].y)>>1);
				ang = getangle(dax-posx,day-posy);

				warpsectorflag = 1;
			}
		}
		else
			warpsectorflag = 0;

		for(i=0;i<xpanningsectorcnt;i++)   //animate wall x-panning sectors
		{
			dasector = xpanningsectorlist[i];

			startwall = sector[dasector].wallptr;
			endwall = startwall+sector[dasector].wallnum-1;
			for(s=startwall;s<=endwall;s++)
				wall[s].panning = ((totalclock>>2)&255);
		}

		for(i=0;i<ypanningwallcnt;i++)
			wall[ypanningwalllist[i]].panning = ((totalclock>>1)&255);

		for(i=0;i<floorpanningcnt;i++)   //animate floor of slime sectors
			sector[floorpanninglist[i]].floorpanning = ((totalclock>>2)&255);

		//Check if you're touching any sprites in your sector...
		i = headspritesect[cursectnum];
		while (i != -1)
		{
			nexti = nextspritesect[i];
			if ((labs(posx-sprite[i].x)+labs(posy-sprite[i].y)) < 512)
			{
				gotsprite = 0;
				switch(sprite[i].picnum)
				{
					case COINSTACK:
						wsay("getstuff.wav",255);
						score += 1000L;
						health += 25;
						deletesprite((short)i);
						gotsprite = 1;
						break;
					case GIFTBOX:
						wsay("getstuff.wav",255);
						score += 1000L;
						health += 10;
						deletesprite((short)i);
						gotsprite = 1;
						break;
					case COIN:
						wsay("getstuff.wav",255);
						score += 100L;
						health += 5;
						deletesprite((short)i);
						gotsprite = 1;
						break;
					case DIAMONDS:
						wsay("getstuff.wav",255);
						score += 500L;
						health += 15;
						deletesprite((short)i);
						gotsprite = 1;
						break;
					case BULLET:
						if ((numplayers == 2) && (sprite[i].owner == otherplayer) && (labs(posx-sprite[i].x)+labs(posy-sprite[i].y) < 768))
						{
							deletesprite((short)i);

							health -= 25;
							if (health <= 0)
							{
								wsay("death.wav",255);
								health = 100;
								nukes[1]++;
								good = 0;
								while (good == 0)
								{
									k = (rand()%numsectors);
									startwall = sector[k].wallptr;
									endwall = startwall+sector[k].wallnum-1;
									if (endwall > startwall)
									{
										dax = 0L;
										day = 0L;
										for(s=startwall;s<=endwall;s++)
										{
											dax += wall[s].x;
											day += wall[s].y;
										}
										dax /= (endwall-startwall+1);
										day /= (endwall-startwall+1);
									}
									if (inside(dax,day,k) == 1)
										good = 1;
								}
								posx = dax;
								posy = day;
								posz = sector[k].floorz-heightofffloor;
								getsector();
								cursectnum = k;

								comsend(7L,1);
								comsend(nukes[1],2);
							}
							else
							{
								wsay("ouch.wav",255);
								comsend(8L,1);
							}
						}
						break;
				}
				if ((numplayers == 2) && (gotsprite == 1))
				{
					comsend(6L,1);
					comsend(i,2);
				}
			}

			i = nexti;
		}

		i = headspritestat[1];     //Go through active sprites
		while (i != -1)
		{
			nexti = nextspritestat[i];

			if (sprite[i].picnum == SKELETON)
			{
				dax = (sintable[(sprite[i].ang+512)&2047]>>9);   //move skeleton
				day = (sintable[sprite[i].ang]>>9);
				if (movesprite(i,dax,day,0L) == 0)
					sprite[i].ang = getangle(posx-sprite[i].x,posy-sprite[i].y);
				else
					sprite[i].z = sector[sprite[i].sectnum].floorz;
			}
			if (sprite[i].picnum == BROWNMONSTER)
			{
				if ((rand()&31) == 0)    //brown monster decides to shoot bullet
				{
					if (cansee(posx,posy,posz,cursectnum,sprite[i].x,sprite[i].y,sprite[i].z-(tilesizy[sprite[i].picnum]<<7),sprite[i].sectnum) == 1)
					{
						wsay("zipguns.wav",1023/((((sprite[i].x-posx)*(sprite[i].x-posx)+(sprite[i].y-posy)*(sprite[i].y-posy))>>21)+4));
						j = insertsprite(sprite[i].sectnum,1);
						sprite[j].x = sprite[i].x;
						sprite[j].y = sprite[i].y;
						sprite[j].z = sector[sprite[i].sectnum].floorz-(24<<8);
						sprite[j].cstat = 0;        //Hitscan does not hit other bullets
						sprite[j].picnum = BULLET;
						sprite[j].shade = 0;
						sprite[j].xrepeat = 64;
						sprite[j].yrepeat = 64;
						sprite[j].ang = (((getangle(posx-sprite[j].x,posy-sprite[j].y)+(rand()&15)-8)+2048)&2047);
						sprite[j].vel = 256;
						sprite[j].zvel = ((posz+(8<<8)-sprite[j].z)<<8)/ksqrt((posx-sprite[j].x)*(posx-sprite[j].x)+(posy-sprite[j].y)*(posy-sprite[j].y));
						sprite[j].owner = i;
						sprite[j].tag = 0L;
					}
				}

				//Move brown monster
				dax = (sintable[(sprite[i].ang+512)&2047]>>8);
				day = (sintable[sprite[i].ang]>>8);
				if (movesprite(i,dax,day,0L) == 0)
					sprite[i].ang = (rand()&2047);
				else
					sprite[i].z = sector[sprite[i].sectnum].floorz;
			}
			if ((sprite[i].picnum == EVILAL) && (sprite[i].yrepeat >= 38))
			{
				if (sprite[i].yrepeat < 64)
				{
					sprite[i].xrepeat++;
					sprite[i].yrepeat++;
				}
				else
				{
					if ((rand()&31) == 0)
					{                   //Al decides to reproduce
						wsay("zipguns.wav",1023/((((sprite[i].x-posx)*(sprite[i].x-posx)+(sprite[i].y-posy)*(sprite[i].y-posy))>>21)+4));
						j = insertsprite(sprite[i].sectnum,1);
						sprite[j].x = sprite[i].x;
						sprite[j].y = sprite[i].y;
						sprite[j].z = 0;
						sprite[j].cstat = 1;         //Hitscan should hit new monsters
						sprite[j].picnum = EVILAL;
						sprite[j].shade = 0;
						sprite[j].xrepeat = 38;
						sprite[j].yrepeat = 38;
						sprite[j].ang = (rand()&2047);
						sprite[j].vel = 0;
						sprite[j].zvel = 0;
						sprite[j].owner = i;
						sprite[j].tag = 0L;
					}
					if ((rand()&31) == 0)     //Al decides to shoot bullet
					{
						if (cansee(posx,posy,posz,cursectnum,sprite[i].x,sprite[i].y,sprite[i].z-(tilesizy[sprite[i].picnum]<<7),sprite[i].sectnum) == 1)
						{
							wsay("zipguns.wav",1023/((((sprite[i].x-posx)*(sprite[i].x-posx)+(sprite[i].y-posy)*(sprite[i].y-posy))>>21)+4));
							j = insertsprite(sprite[i].sectnum,1);
							sprite[j].x = sprite[i].x;
							sprite[j].y = sprite[i].y;
							sprite[j].z = sector[sprite[i].sectnum].floorz-(24<<8);
							sprite[j].cstat = 0;            //Hitscan does not hit other bullets
							sprite[j].picnum = BULLET;
							sprite[j].shade = 0;
							sprite[j].xrepeat = 64;
							sprite[j].yrepeat = 64;
							sprite[j].ang = (((getangle(posx-sprite[j].x,posy-sprite[j].y)+(rand()&15)-8)+2048)&2047);
							sprite[j].vel = 256;
							sprite[j].zvel = ((posz+(8<<8)-sprite[j].z)<<8)/ksqrt((posx-sprite[j].x)*(posx-sprite[j].x)+(posy-sprite[j].y)*(posy-sprite[j].y));
							sprite[j].owner = i;
							sprite[j].tag = 0L;
						}
					}

					//Move Al
					dax = (sintable[(sprite[i].ang+512)&2047]>>7);
					day = (sintable[sprite[i].ang]>>7);
					if (movesprite(i,dax,day,0L) == 0)
					{
						if ((rand()&1) == 0)
							sprite[i].ang = getangle(posx-sprite[i].x,posy-sprite[i].y);
						else
							sprite[i].ang = (rand()&2047);
					}
					else
						sprite[i].z = sector[sprite[i].sectnum].floorz;
				}
			}

				 //If the sprite is a bullet then...
			if (sprite[i].picnum == BULLET)
			{
				dax = (((long)sprite[i].vel*(long)sintable[(sprite[i].ang+512)&2047])>>14);
				day = (((long)sprite[i].vel*(long)sintable[sprite[i].ang])>>14);
				daz = sprite[i].zvel;

				if (movesprite(i,dax,day,daz) == 0)
				{
						//If the bullet was unable to move (ex: hits a wall)
					wsay("bullseye.wav",1023/((((sprite[i].x-posx)*(sprite[i].x-posx)+(sprite[i].y-posy)*(sprite[i].y-posy))>>21)+4));
					deletesprite((short)i);
					goto spriteisdeletedskip;
				}
				else
				{
					dax = sprite[i].x;
					day = sprite[i].y;
					daz = sprite[i].z;

					if ((daz < sector[sprite[i].sectnum].ceilingz) || (daz > sector[sprite[i].sectnum].floorz))
					{
						//If the bullet hits a ceiling / floor
						wsay("bullseye.wav",1023/((((dax-posx)*(dax-posx)+(day-posy)*(day-posy))>>21)+4));
						deletesprite((short)i);
						goto spriteisdeletedskip;
					}

					if ((labs(posx-dax)+labs(posy-day) < 256) && (sprite[i].owner != -1) && (sprite[i].owner != otherplayer))
					{
						  //If the bullet hits you
						wsay("ouch.wav",255);
						deletesprite((short)i);
						goto spriteisdeletedskip;
					}
					else
					{
						//Check to see it the bullet hits any monsters in the
						//current sector by scanning through the linked list
						//of all sprites in the bullet's sector

						j = headspritesect[sprite[i].sectnum];
						while (j >= 0)
						{
							if (sprite[i].owner != j)
								if ((labs(sprite[j].x-dax)+labs(sprite[j].y-day)) < 512)
								{
									if (sprite[j].picnum == BROWNMONSTER)
									{
										wsay("blowup.wav",1023/((((dax-posx)*(dax-posx)+(day-posy)*(day-posy))>>21)+4));
										sprite[j].picnum = GIFTBOX;
										changespritestat(j,0);
										if (sprite[i].owner == -1)
											score += 200L;
										deletesprite((short)i);
										goto spriteisdeletedskip;
									}
									if (sprite[j].picnum == DOOMGUY)
									{
										wsay("blowup.wav",1023/((((dax-posx)*(dax-posx)+(day-posy)*(day-posy))>>21)+4));
										sprite[j].picnum = SKELETON;
										if (sprite[i].owner == -1)
											score += 200L;
										deletesprite((short)i);
										goto spriteisdeletedskip;
									}
									if (sprite[j].picnum == EVILAL)
									{
										wsay("blowup.wav",1023/((((dax-posx)*(dax-posx)+(day-posy)*(day-posy))>>21)+4));
										sprite[j].picnum = EVILALGRAVE;
										sprite[j].xrepeat = (rand()&63)+16;
										sprite[j].yrepeat = sprite[j].xrepeat;
										if (sprite[i].owner == -1)
											score += 200L;
										deletesprite((short)i);
										goto spriteisdeletedskip;
									}
									if (sprite[j].picnum == AL)
									{
										wsay("blowup.wav",1023/((((dax-posx)*(dax-posx)+(day-posy)*(day-posy))>>21)+4));
										sprite[j].xrepeat += 2;
										sprite[j].yrepeat += 2;
										if (sprite[j].yrepeat >= 38)
											sprite[j].picnum = EVILAL;
										changespritestat(j,1);
										deletesprite((short)i);
										goto spriteisdeletedskip;
									}
								}
							j = nextspritesect[j];
						}
					}
				}
			}

spriteisdeletedskip:
			i = nexti;
		}

		i = headspritestat[2];     //Go through monster waiting for you list
		while (i != -1)
		{
			nexti = nextspritestat[i];
			if (cansee(posx,posy,posz,cursectnum,sprite[i].x,sprite[i].y,sprite[i].z-(tilesizy[sprite[i].picnum]<<7),sprite[i].sectnum) == 1)
			{
				changespritestat(i,1);
				wsay("iseeyou.wav",1023/((((sprite[i].x-posx)*(sprite[i].x-posx)+(sprite[i].y-posy)*(sprite[i].y-posy))>>21)+4));
			}

			i = nexti;
		}

	}
	if (numplayers == 2)
	{
		comsend(255L,1);
	}

	musicoff();
	uninitengine();
	uninitsb();
	setvmode(0x3);
	showengineinfo();
	return(0);
}

/****************************************************************************
Ŀ
                        IMPORTANT ENGINE FUNCTIONS:                        


initengine();
			 Sets up interrupt vectors for keyboard and timer, and initializes
			 many variables for the BUILD engine.  You should call this once
			 before any other functions of the BUILD engine are used.
uninitengine();
			 Restores interrupt vectors for keyboard and timer, and frees
			 buffers.  You should call this once at the end of the program
			 before quitting to dos.
loadboard(char filename[20]);
			 Loads the given board file into memory for the BUILD engine.
			 Returns -1 if file not found.  If no extension is given, .MAP will
			 be appended to the filename.
saveboard(char filename[20]);
			 Saves the given board from memory inro the specified filename.
			 Returns -1 if unable to save.  If no extension is given, .MAP will
			 be appended to the filename.
loadpics(char filename[20]);
			 Loads the given artwork file into memory for the BUILD engine.
			 Returns -1 if file not found.  If no extension is given, .ART will
			 be appended to the filename.
setgamemode();
			 This function sets the video mode to 320*200*256color graphics.
			 Since BUILD supports several different modes including mode x,
			 mode 13h, and other special modes, I don't expect you to write
			 any graphics output functions.  (Soon I have all the necessary
			 functions)  If for some reason, you use your own graphics mode,
			 you must call this function again before using the BUILD drawing
			 functions.
draw3dscreen();
			 This function draws the 3D screen to the current drawing page,
			 which is not yet shown.  This way, you can overwrite some things
			 over the 3D screen such as a gun.  To view the screen, use the
			 nextpage() function.  The nextpage() function should always be
			 called sometime after each draw3dscreen() function.
engineinput();
			 This function allows the engine to adjust your position depending
			 on the status of the arrow keys, and other control keys.  It
			 handles timing and clipping.
nextpage();
			 After a screen is prepared, use this function to view the screen.

Ŀ
                           OTHER ENGINE FUNCTIONS:                         


overwritesprite (long x, long y, short tilenum, char shade, char orientation);
			 Use this function to write sprites over the 3d view.  For example,
			 the menu system in DOOM can be done with this function.  Be sure
			 that you call this function for every single frame after the 3d
			 view is drawn or else it will be flashed on for only 1 frame.
			 If you want x and y to be the top left corner, set the orientation
			 to 0.  If you want x and y to be the middle of the sprite, set the
			 orientation to 1.  The reason I included the orienation = 1 option
			 is so that if you want a sprite centered and the size of the tile
			 changes, you don't need to recompile and guess where the new top
			 left corner is.  Oh yeah, and I forget to mention that if shade is
			 greater than 32, than overwritesprite does transluscence.  (Try it
			 out!)

permanentwritesprite (long x, long y, short tilenum, char shade);
			- Added permanentwritesprite function for status bars or other
				  sections of the screen that will not be overwritten by the
				  engine.  When using this function, you may want to modify the
				  STARTUMOST / STARTDMOST arrays. The format of this function
				  is like overwritesprite except that the x and y are always top
				  left corner, no orientation variable, and no translucence.
				  Here's an example that will draw a 320*32 status bar at the
				  bottom of the screen:
					  permanentwritesprite(0,YDIM-32,STATUSBAR1,0);
					  permanentwritesprite(128,YDIM-32,STATUSBAR2,0);
					  permanentwritesprite(256,YDIM-32,STATUSBAR3,0);
				  This will change when EDITART supports any size tiles.

printext(long x, long y, char buffer[42], short tilenum, char invisiblecol);
			 Use this function to print text anywhere on the screen from a font
			 that you can create in EDITART.  Please see my example font in
			 TILES.ART to see how I lay out the user-defined font.  X ranges
			 from 0-319. Y ranges from 0-199.  The buffer is the string to
			 print.  Tilenum specifies which font to use.  Invisiblecol tells
			 printext what color to draw the transparent pixels.  If
			 invisiblecol is 255 then the transpararent pixels are still
			 transparent.
printnum(long x, long y, long num, short tilenum, char invisiblecol);
			 Printnum is a function call that will print a long integer (num)
			 starting at top left corner x, y.  Please look at the documentation
			 for printext, since internally, printnum simply prepares a buffer
			 and calls the printext function.

setvmode(long videomode);
			 If you look at the top of GAME.C, you will see something like this:
			 #pragma aux setvmode =\...  This is how you do in-line assembler in
			 WATCOM C.  All this function is doing is setting the video mode.
showengineinfo();
			 Use this function after setting to text mode to view some statics
			 about the engine, such as frame rate.
resettiming();
			 Resets timing, such as setting totalclock = 0.  Also resets other
			 timers.  This is for use with the showengineinfo function above.

ksqrt(long num);  returns (long)square root
			 A square root function optimized for integers.  Use this function
			 only if you want to.

getangle(long xvect,long yvect);    returns (short)angle;
			 Use this function call to determine the angle between two points.
			 For example, if you want a monster to shoot a bullet towards you,
			 you would get the bullet's angle this way:
	sprite[bullet].ang = getangle(posx-sprite[monst].x,posy-sprite[monst].y);

clipmove (long vel, short ang);
	Moves player forward / backward by the velocity (vel) passed to it.  This
	function also clips the player from going through walls.  Please use this
	function instead of adding the cos/sin to posx/posy yourself.

inside(long x, long y, short sectnum);
	Tests to see whether the overhead point (x, y) is inside sector (sectnum)
	Returns either 0 or 1, where 1 means it is inside, and 0 means it is not.

copytilepiece(long walnume1, long x1, long y1, long xsiz, long ysiz,
 long walnume2, long x2, long y2, char shadeoffs);
	Copies section of tile 1 (walnume1) with top-left corner (x1,y1) and
	rectangular size (xsiz, ysiz) to top-left corner (x2, y2) of tile 2
	(walnume).  You can animate tiles with this function.  For example, with
	this function, you can make a slot machine like in Ken's Labyrinth or an
	electronic sign with text sliding from right to left.

hitscan(long x, long y, long z, short daang, short sectnum, long range,
	  long *hitx, long *hity, long *hitsect, long *hitwall, long *hitsprite);
	Ray-casts starting at coordinate (x,y,z) and sector (sectnum), and shoots
	the ray at angle (daang).  This function finds the first point at which
	the ray hits either a wall or a sprite.  It always returns the x, y, and
	sector (hitx, hity, hitsect), but the ray can hit either a wall or a
	sprite, not both.  So either hitwall or hitsprite will be a -1.


cansee(long x1, long y1, long z1, short sectnum1,
		 long x2, long y2, long z2, short sectnum2);  returns 0 or 1
	This function determines whether or not two 3D points can "see" each
	other or not.  All you do is pass it the coordinates of a 3D line defined
	by two 3D points (with their respective sectors)  The function will return
	a 1 if the points can see each other or a 0 if there is something blocking
	the two points from seeing each other.  This is how I determine whether a
	monster can see you or not. Try playing DOOM1.DAT to fully enjoy this
	great function!

setanimation(long *animptr, long thegoal, long thevel);
	This is a function for your convenience that will animate a long
	variable, such as sector[].floorz for platforms, or sector[].ceilingz
	for doors.  All you do is pass it the long pointer into memory, specifying
	which long variable is to be animated; you also pass the goal (long value
	to animate towards), and the velocity at which the variable is animated.
	Velocity = 128 is a normal speed door.  You may also modify the animation
	arrays directly if you wish:

	The animation arrays are as follows:
		long *animateptr[MAXANIMATES], animategoal[MAXANIMATES];
		long animatevel[MAXANIMATES], animatecnt;

getanimationgoal(long animptr);
	Check to see if a certain variable in memory is already being animated
	by the engine.  If so, an index into the animation arrays is returned,
	else -1 is returned.  This is function is useful when you are press
	space bar near a door, and it is already animating, you simply want
	to reverse its direction.

nextsectorneighborz(short sectnum, long thez, short topbottom, short direction);
	This function searches z-coordinates of neighboring sectors to find the
	closest (next) ceiling starting at the given z-coordinate (thez).
	For example, if you want to find the goal z-coordinate when opening a
	door, you might want the door to stop at the next closest neighboring
	ceiling z-coordinate.  You can get the z-coordinate this way:

		newz = sector[nextsectorneighborz(sectnum,startz,-1,-1)].ceilingz

	topbottom (3rd parameter)  -1 = search ceilings
										 1 = search floors
	direction (4th parameter)  -1 = search upwards
										 1 = search downwards

playposcapt(char filename[20]);
	Plays the recorded run previosly recorded (such as POSCAPT.DAT).
	-Not a very useful yet.
screencapture();
	Capture the screen and save it as a .BMP file.  I don't know why my
	.BMP format isn't compatible with other programs.
loadpcx(char filename[20], long ptrlong);
	Loads a .PCX image into a given region in memory.  To go directly to
	video memory, set ptrlong = 0xa0000.  It is probably easier just to
	put the image into TILES.ART and use the overwritesprite function.
loadbmp(char filename[20], long ptrlong);
	Loads a .PCX image into a given region in memory.  To go directly to
	video memory, set ptrlong = 0xa0000.  It is probably easier just to
	put the image into TILES.ART and use the overwritesprite function.

Ŀ
                             SPRITE FUNCTIONS:                             


insertsprite(short sectnum, short statnum);   //returns (short)spritenum;
	Whenever you insert a sprite, you must pass is the sector
	number, and a status number (statnum).  The status number can be any
	number from 0 to MAXSTATUS-1.  Insertsprite works like a memory
	allocation function and returns the sprite number.

deletesprite(short spritenum);
	Deletes the sprite.

changespritesect(short spritenum, short newsectnum);
	Changes the sector of sprite (spritenum) to the
	newsector (newsectnum).  This function may become
	internal to the engine in the movesprite function.  But
	this function is necessary since all the sectors have
	their own doubly-linked lists of sprites.

changespritestat(short spritenum, short newstatnum);
	Changes the status of sprite (spritenum) to status
	(newstatus).  Newstatus can be any number from 0 to MAXSTATUS-1.
	You can use this function to put a monster on a list of active sprites
	when it first sees you.

movesprite(short spritenum, long xchange, long ychange, long zchange);
	 returns 0 - did not move (hit wall), 1 - moved successfully;
			 This function moves the sprite given by spritenum by the 3
			 increments, xchange, ychange, and zchange.  If the sprite hits
			 a wall, then the sprite's position does not change and a 0 is
			 returned.

Ŀ
                         SERIAL PORT FUNCTIONS:                            

Notes: These functions may be rewritten to be easier to use in the future, or
	when there is support for other methods of communcation.

comon();
	Initializes the com port if a com port is enabled in the setup program.
comoff();
	Uninitializes the com port.  You must call this before exiting your
	program if you called comon.
comsend(long dat, long numbytes);
	Sends 1, 2, 3, or 4 bytes of data to the com port.
	To send a char, do this:       comsend((long)mychar,1L);
	To send a short int, do this:  comsend((long)myshort,2L);
	To send a long int, do this:   comsend(mylong,4L);
comgetchar();
	Comgetchar attempts to receive 1 character.  If no characters are ready,
	then instead of waiting until one is ready, comgetchar returns a -1, else
	a number from 0-255 is returned.

Ŀ
                       DIGITIZED SOUND FUNCTIONS:                          

Note: If you want to write your own digitized sound driver, simply delete
	these functions from GAME.C.

initsb(char sndspeed);
			 Initializes the digitized sound routines.  You need to call this
			 only once at the beginning of the program.  The speed number is
			 determined by this equation:
				 speed# = 256 - (1,000,000 / sample_rate)
			 Currently, the sample_rate is around 11,000 Hertz. (speed# = 165)
wsay(char filename[20], char volume);
			 Play the sound file (filename[20]) at the given volume.  Volume
			 ranges from 0 (silent) to 255 (full volume).  Here is a good
			 equation for calculating sound volume from point (x1,y1) to
			 (x2,y2):
		wsay("blowup.wav",1023/(( ((x1-x2)-(x1-x2)+(y1-y2)*(y1-y2)) >>21)+4));

uninitsb();
			 Turns the speaker off, so sounds don't continue playing while
			 your back in DOS.

Ŀ
                            MUSIC FUNCTIONS:                               

Note: If you want to write your own music driver, simply delete these
	functions from GAME.C.  The BUILD engine uses interrupt vector 0x8 (IRQ0)
	and that may complicate things.  If you like my music, perhaps I can send
	you my MIDI sequencer program (It requires MPU-401, and TSENG-ET4000 SVGA,
	but could be standardized if there's enough demand).

loadmusic(char filename[20]);
			 Loads the given song into memory.  Be sure INSTS.DAT is in the
			 current directory.  If no extension is given, then .KSM will be
			 appended to the filename.  You should use this function only when
			 the music is off.
musicon();
			 Enable the playing of music.  Use this only after loadmusic has
			 been called.
musicoff();
			 Disable the playing of music.  Be sure to call this before quitting
			 to DOS.
****************************************************************************/
